package org.bitbucket.bradleysmithllc.etlunit.io;

import java.io.FilterOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.concurrent.Semaphore;

public class SynchronizedFilterOutputStream extends FilterOutputStream
{
	private final Semaphore lock;

	public SynchronizedFilterOutputStream(OutputStream out, Semaphore lock)
	{
		super(out);

		this.lock = lock;
	}

	@Override
	public final void write(int b) throws IOException
	{
		lock.acquireUninterruptibly();

		try
		{
			out.write(b);
		}
		finally
		{
			lock.release();
		}
	}

	@Override
	public final void write(byte[] b) throws IOException
	{
		lock.acquireUninterruptibly();

		try
		{
			out.write(b);
		}
		finally
		{
			lock.release();
		}
	}

	@Override
	public final void write(byte[] b, int off, int len) throws IOException
	{
		lock.acquireUninterruptibly();

		try
		{
			out.write(b, off, len);
		}
		finally
		{
			lock.release();
		}
	}

	@Override
	public final void flush() throws IOException
	{
		lock.acquireUninterruptibly();

		try
		{
			out.flush();
		}
		finally
		{
			lock.release();
		}
	}

	@Override
	public final void close() throws IOException
	{
		lock.acquireUninterruptibly();

		try
		{
			out.close();
		}
		finally
		{
			lock.release();
		}
	}
}