package org.bitbucket.bradleysmithllc.etlunit.parser;

import java.util.*;

public class ETLTestAnnotatedImpl extends ETLTestDebugTraceableImpl implements ETLTestAnnotated
{
	List<ETLTestAnnotation> annotations = new ArrayList<ETLTestAnnotation>();
	Map<String, List<ETLTestAnnotation>> annotationMap = new HashMap<String, List<ETLTestAnnotation>>();

	public ETLTestAnnotatedImpl(Token token)
	{
		super(token);
	}

	public ETLTestAnnotatedImpl()
	{
	}

	public void addAnnotations(List<ETLTestAnnotation> ano)
	{
		Iterator<ETLTestAnnotation> it = ano.iterator();

		while (it.hasNext())
		{
			addAnnotation(it.next());
		}
	}

	public void addAnnotation(ETLTestAnnotation ano)
	{
		annotations.add(ano);

		List<ETLTestAnnotation> li = annotationMap.get(ano.getName());

		if (li == null)
		{
			li = new ArrayList<ETLTestAnnotation>();
			annotationMap.put(ano.getName(), li);
		}

		li.add(ano);
	}

	@Override
	public List<ETLTestAnnotation> getAnnotations()
	{
		return Collections.unmodifiableList(annotations);
	}

	@Override
	public List<ETLTestAnnotation> getAnnotations(String name)
	{
		List<ETLTestAnnotation> list = annotationMap.get(name);

		if (list == null)
		{
			return Collections.EMPTY_LIST;
		}

		return Collections.unmodifiableList(list);
	}

	@Override
	public boolean hasAnnotation(String name)
	{
		return annotationMap.containsKey(name);
	}

	public String getDescription()
	{
		if (!hasAnnotation("@Description"))
		{
			return "";
		}

		List<ETLTestAnnotation> desc = getAnnotations("@Description");

		ETLTestAnnotation dAnn = desc.get(0);

		if (!dAnn.hasValue())
		{
			throw new IllegalStateException("@Description annotation missing description attribute");
		}
		else if (dAnn.getValue().getValueType() != ETLTestValueObject.value_type.object)
		{
			throw new IllegalStateException("@Description annotation must have an object type argument");
		}

		ETLTestValueObject val = dAnn.getValue().getValueAsMap().get("description");

		if (val == null)
		{
			throw new IllegalStateException("@Description annotation missing description attribute");
		}
		else if (val.getValueType() != ETLTestValueObject.value_type.quoted_string)
		{
			throw new IllegalStateException("@Description description attribute must be text");
		}

		return val.getValueAsString();
	}
}